# Copyright (c) 2017 Civic Knowledge. This file is licensed under the terms of the
# Revised BSD License, included in this distribution as LICENSE

"""
CLI program for storing pacakges in CKAN

The program uses the Root.Distributions in the source package to locate packages to link into a CKAN record.

"""

from metapack.cli.core import prt
from metapack.package import *
from .core import MetapackCliMemo as _MetapackCliMemo

downloader = Downloader()

class MetapackCliMemo(_MetapackCliMemo):

    def __init__(self, args, downloader):
        super().__init__(args, downloader)
        self


def info_args(subparsers):

    parser = subparsers.add_parser(
        'info',
        help='Print info about a package '
    )

    parser.set_defaults(run_command=info)

    group = parser.add_mutually_exclusive_group()

    group.add_argument('-n', '--name', default=False, action='store_true',
                             help="Print the name, with version")
    group.add_argument('-N', '--root-name', default=False, action='store_true',
                       help="Print the name, without the version")

    parser.add_argument('metatabfile', nargs='?',
                        help="Path or URL to a metatab file. If not provided, defaults to 'metadata.csv' ")

def info(args):

    m = MetapackCliMemo(args, downloader)

    if m.args.name:
        prt(m.doc.name)
    elif m.args.root_name:
        prt(m.doc.as_version(None))


# Copyright (c) 2017 Civic Knowledge. This file is licensed under the terms of the
# Revised BSD License, included in this distribution as LICENSE

"""
CLI program for storing pacakges in CKAN

The program uses the Root.Distributions in the source package to locate packages to link into a CKAN record.

"""

from os import listdir
from os.path import join, exists

from metapack.package import *
from rowgenerators import FileUrl, RowGeneratorError, parse_app_url
from .core import MetapackCliMemo as _MetapackCliMemo
from .core import err, PACKAGE_PREFIX, add_package_to_index, prt
import dbm
import json

downloader = Downloader()



class MetapackCliMemo(_MetapackCliMemo):

    def __init__(self, args, downloader):
        super().__init__(args, downloader)

def new_args(subparsers):

    parser = subparsers.add_parser(
        'new',
        help='Create new Metatab packages'
    )

    parser.set_defaults(run_command=new_cmd)

    parser.add_argument('-o', '--origin', help="Dataset origin, usually a second-level domain name")
    parser.add_argument('-d', '--dataset', help="Main dataset name", required = True)
    parser.add_argument('-t', '--time', help="Temporal extents")
    parser.add_argument('-s', '--space', help="Space, geographic extent")
    parser.add_argument('-g', '--grain', help="Grain, the type of entity a row represents")
    parser.add_argument('-v', '--variant', help="Variant, any distinguishing string")
    parser.add_argument('-r', '--revision', help="Version, defaults to 1", default=1)

    parser.add_argument('-T', '--template', help="Metatab file template, defaults to 'metatab' ", default='metatab')
    parser.add_argument('-C', '--config', help="Path to config file. Defaults to ~/.metapack-defaults.csv or value of METAPACK_DEFAULTS env var")

def new_cmd(args):

    from metapack import MetapackDoc
    from metapack.util import make_metatab_file, datetime_now, ensure_dir
    from metapack.cli.core import write_doc, prt, err, warn
    from os.path import exists, join, expanduser
    from metatab import DEFAULT_METATAB_FILE
    from os import getenv

    if args.config:
        config_file = args.config
    elif getenv("METAPACK_CONFIG"):
        config_file = getenv("METAPACK_DEFAULTS")
    elif expanduser('~/.metapack-default.csv'):
        config_file = expanduser('~/.metapack-defaults.csv')
    else:
        config_file = None

    if config_file and exists(config_file):
        prt(f"Using defaults file '{config_file}'")
        config = MetapackDoc(config_file)
    else:
        config = MetapackDoc()

    doc = make_metatab_file(args.template)

    doc['Root']['Created'] = datetime_now()

    origin = args.origin or config.get_value('Root.Origin')

    if not origin:
        err("Must specify a value for origin, either on command line or in defaults file")

    doc['Root'].find_first('Root.Origin').value = origin
    doc['Root'].find_first('Root.Dataset').value = args.dataset
    doc['Root'].find_first('Root.Space').value = args.space or config.get_value('Root.Space')
    doc['Root'].find_first('Root.Time').value = args.time or config.get_value('Root.Time')
    doc['Root'].find_first('Root.Grain').value = args.grain or config.get_value('Root.Grain')
    doc['Root'].find_first('Root.Variant').value = args.variant or config.get_value('Root.Variant')
    doc['Root'].find_first('Root.Version').value = args.revision or config.get_value('Root.Version')

    # Copy contacts in
    if 'Contacts' in config:
        for c in config['Contacts']:
            doc['Contacts'].add_term(c)

    nv_name = doc.as_version(None)

    if exists(nv_name):
        err(f"Directory {nv_name} already exists")

    ensure_dir(nv_name)

    prt(f"Writing to '{nv_name}'")

    write_doc(doc, join(nv_name, DEFAULT_METATAB_FILE))



# Copyright (c) 2017 Civic Knowledge. This file is licensed under the terms of the
# MIT License, included in this distribution as LICENSE

from metapack import Downloader
from metapack.cli.core import prt, err, MetapackCliMemo
from metapack.appurl import SearchUrl
from rowgenerators import AppUrlError, parse_app_url

downloader = Downloader()

def search(subparsers):

    parser = subparsers.add_parser(
        'search',
        help='Search for packages',
        epilog='Cache dir: {}\n'.format(str(downloader.cache.getsyspath('/'))))

    parser.add_argument('-l', '--list', default=False, action='store_true',
                        help="List the packages that would be indexed ( Only from the JSON index")

    parser.set_defaults(run_command=run_search)

    parser.add_argument('search', nargs='?', help="Path or URL to a metatab file")

def run_search(args):

    if not args.search or args.list:
        from .core import get_search_index
        from tabulate import tabulate

        print(tabulate(sorted([(k,v['type'],v['url']) for k,v in get_search_index().items()]),
                                headers='Key Type Url'.split()))

    else:

        if args.search.startswith('index'):
            url_s = args.search
        else:
            url_s = 'index:'+args.search

        try:
            u = parse_app_url(url_s)

            prt(str(u.get_resource()))

        except AppUrlError as e:
            err(f"Failed to resolve: {str(u)}; {str(e)}")






# Copyright (c) 2017 Civic Knowledge. This file is licensed under the terms of the
# Revised BSD License, included in this distribution as LICENSE

"""
CLI program for storing pacakges in CKAN

The program uses the Root.Distributions in the source package to locate packages to link into a CKAN record.

"""

from os import listdir
from os.path import join, exists

from metapack.package import *
from rowgenerators import FileUrl, RowGeneratorError, parse_app_url
from .core import MetapackCliMemo as _MetapackCliMemo
from .core import err, PACKAGE_PREFIX, add_package_to_index, prt
import dbm
import json

downloader = Downloader()



class MetapackCliMemo(_MetapackCliMemo):

    def __init__(self, args, downloader):
        super().__init__(args, downloader)
        self


def index_args(subparsers):

    parser = subparsers.add_parser(
        'index',
        help='Index packages for searching. '
    )

    parser.set_defaults(run_command=index)

    parser.add_argument('-l', '--list', default=False, action='store_true',
                             help="List the packages that would be indexed")

    parser.add_argument('-d', '--directory', default=downloader.cache.getsyspath('/'),
                        help="Directory where index will be stored")

    parser.add_argument('metatab_url', nargs='?', default='./',
                        help='URL to a metatab package or container for packages')

def try_open(path):
    try:
        p = open_package(path)

        if p.get_value('Root.Issued'):
            yield p
        else:
            if exists(join( p.ref.dirname().path, PACKAGE_PREFIX)):
                yield from yield_packages(parse_app_url((join( p.ref.dirname().path, PACKAGE_PREFIX))))

    except RowGeneratorError as e:
        return

def yield_packages(u):

    if u.isdir():
        yield from try_open(u.path)
        for e in listdir(u.path):
            yield from try_open(join(u.path,e))
    else:
        yield from try_open(u)

def yield_unique_packages(u):
    seen = set()

    for p in yield_packages(u):
        if str(p.ref) not in seen:
            yield p
            seen.add(str(p.ref))

def index(args):

    index_file = join(args.directory, 'index.json')

    try:
        with open(index_file,'r') as f:
            db = json.load(f)
    except (FileNotFoundError, json.JSONDecodeError):
        db = {}

    prt('Index file:', index_file)

    u = parse_app_url(args.metatab_url)

    if not isinstance(u, FileUrl):
        err(f"Can only index File urls, not {type(u)}")

    key_vals = []

    for p in yield_unique_packages(u):
        keys = add_package_to_index(p, db)
        for key in keys:
            key_vals.append((key, p.package_url))

    with open(index_file,'w') as f:
        json.dump(db, f, indent=4)

    from tabulate import tabulate
    print(tabulate(key_vals))